@@ -6,7 +6,7 @@ Similar for the dataset_id, except that the first 8 fields (starting with featur
 """
 from datetime import datetime, date
 from typing import List, Optional, Dict, Union
-from pydantic import BaseModel, Field
+from pydantic import BaseModel, Field, HttpUrl
 # from hashlib import blake2b
 
 
@@ -14,6 +14,20 @@ from pydantic import BaseModel, Field
 ### Models
 
 
+class ConnectionConfig(BaseModel):
+    service_name: str
+    endpoint_url: HttpUrl
+    aws_access_key_id: str
+    aws_secret_access_key: str
+
+
+class Remote(BaseModel):
+    bucket: str
+    connection_config: Union[ConnectionConfig, HttpUrl]
+    version: int = None
+    description: str = Field(None, description='Description of the datasets that are contained within the remote.')
+
+
 class ResultsDimensions(BaseModel):
     lat: int = None
     lon: int = None
@@ -22,6 +36,14 @@ class ResultsDimensions(BaseModel):
     time: int
 
 
+class TimeRange(BaseModel):
+    """
+
+    """
+    from_date: datetime
+    to_date: datetime
+
+
 class Stats(BaseModel):
     """
     Statistics related to the results.
@@ -31,8 +53,6 @@ class Stats(BaseModel):
     mean: float
     median: float
     count: int
-    from_date: datetime
-    to_date: datetime
 
 
 class Geometry(BaseModel):
@@ -58,20 +78,6 @@ class StationBase(BaseModel):
     """
     Contains the base station data.
     """
-    station_id: str = Field(..., description='station uuid based on the geometry')
-    ref: str = None
-    name: str = None
-    osm_id: int = None
-    geometry: Geometry
-    altitude: float = None
-    dimensions: ResultsDimensions = None
-    properties: Dict = Field(None, description='Any additional station specific properties.')
-
-
-class Station(BaseModel):
-    """
-    Contains the station data of a dataset.
-    """
     dataset_id: str = Field(..., description='The unique dataset uuid.')
     station_id: str = Field(..., description='station uuid based on the geometry')
     ref: str = None
@@ -79,39 +85,30 @@ class Station(BaseModel):
     osm_id: int = None
     geometry: Geometry
     altitude: float = None
+    dimensions: ResultsDimensions
+    # heights: Union[List[float], List[int]]
+    heights: List[int]
+    time_range: TimeRange = Field(..., description='The maximum time range of the result.')
     stats: Stats
-    dimensions: ResultsDimensions = None
+    properties: Dict = Field(None, description='Any additional station specific properties.')
+
+
+class Station(StationBase):
+    """
+    Contains the complete station data.
+    """
     results_object_key: List[S3ObjectKey]
-    properties: Dict = Field(None, description='Any additional station specific properties.')
     modified_date: datetime = Field(..., description='The modification date of the last edit.')
 
 
-class StationAgg(BaseModel):
+class StationAgg(StationBase):
     """
-    Contains the station data of a dataset.
+    Contains the complete station data for the combined stations in the entire dataset.
     """
-    dataset_id: str = Field(..., description='The unique dataset uuid.')
-    station_id: str = Field(..., description='station uuid based on the geometry')
-    ref: str = None
-    name: str = None
-    osm_id: int = None
-    geometry: Geometry
-    altitude: float = None
-    stats: Stats
-    dimensions: ResultsDimensions = None
     results_object_key: S3ObjectKey
-    properties: Dict = Field(None, description='Any additional station specific properties.')
     modified_date: datetime = Field(..., description='The modification date of the last edit.')
 
 
-class TimeRange(BaseModel):
-    """
-
-    """
-    from_date: datetime
-    to_date: datetime
-
-
 class Dataset(BaseModel):
     """
     Dataset data.
@@ -135,6 +132,7 @@ class Dataset(BaseModel):
     extent: Geometry = Field(None, description='The geographical extent of the datset as a simple rectangular polygon.')
     time_range: TimeRange = Field(None, description='The maximum time range of the dataset.')
     spatial_resolution: float = Field(None, description='The spatial resolution in decimal degrees if the spatial_distribution is grid.')
+    heights: List[int] = Field(None, description='The heights from all available results in the dataset.')
     cf_standard_name: str = Field(None, description='The CF conventions standard name for the parameter.')
     wrf_standard_name: str = Field(None, description='The WRF standard name for the parameter.')
     precision: float = Field(None, description='The decimal precision of the result values.')
