@@ -6,4 +6,4 @@ Created on Wed Aug 25 09:43:25 2021
 @author: mike
 """
 from .grid import Grid, Processor
-from . import processors
\ No newline at end of file
+# from . import processors
