@@ -11,6 +11,7 @@ import numpy as np
 import xarray as xr
 import pandas as pd
 from tethys_utils.titan import Titan
+from tethys_utils.misc import parse_data_paths
 # from shapely.geometry import shape, mapping, Point, box
 import copy
 # import rasterio
@@ -35,7 +36,7 @@ base_encoding = {'lon': {'dtype': 'int32', '_FillValue': -999999, 'scale_factor'
 ### Functions
 
 
-def split_grid(ds, x_size=None, y_size=None, n_intervals=None, x_name='lon', y_name='lat', mbytes_size=1500):
+def split_grid(ds, x_size=None, y_size=None, n_intervals=None, x_name='lon', y_name='lat', mbytes_size=2000):
     """
     Function to split an n-dimensional dataset along the x and y dimensions. Optionally, add time and height dimensions if the array does not aready contain them.
 
@@ -97,16 +98,28 @@ def split_grid(ds, x_size=None, y_size=None, n_intervals=None, x_name='lon', y_n
     return block_list
 
 
-def select_sample_points(grid, samples=10):
+def select_sample_points(grid, block_size=10):
     """
 
     """
-    g1 = grid.isel(time=0, height=0, drop=True)
-    df1 = g1.to_dataframe().dropna().reset_index()
-    r1 = np.random.choice(range(len(df1)), samples, False)
-    df2 = df1.iloc[r1][['lon', 'lat']].reset_index(drop=True).copy()
+    g1 = grid.isel(time=0, height=0, drop=True).copy().load()
+    var1 = [v for v in list(g1.variables) if not v in ['lat', 'lon']][0]
 
-    return df2
+    blocks = split_grid(g1, block_size, block_size)
+    blocks1 = [b for b in blocks if int(b[var1].isnull().sum()) == 0]
+
+    r1 = np.random.choice(range(len(blocks1)), 1)[0]
+
+    block = blocks1[r1]
+
+    lons = block['lon'].values
+    lats = block['lat'].values
+
+    # df1 = g1.to_dataframe().dropna().reset_index()
+    # r1 = np.random.choice(range(len(df1)), samples, False)
+    # df2 = df1.iloc[r1][['lon', 'lat']].reset_index(drop=True).copy()
+
+    return lons, lats
 
 
 def grid_checks(dims):
@@ -123,21 +136,30 @@ def grid_checks(dims):
         raise ValueError("height must be a dimension.")
 
 
-def iter_obj_sizes(pts_df, grid, encoding):
+def iter_obj_sizes(lons_lats, grid, encoding):
     """
 
     """
-    obj_sizes = []
-    for i, row in pts_df.iterrows():
-        # print(i)
-        val1 = grid.sel(lon=row['lon'], lat=row['lat']).copy()
-        val1.load()
-        for k, v in encoding.items():
+    lons, lats = lons_lats
+
+    block_size = len(lons)
+
+    val1 = grid.sel(lon=lons, lat=lats).copy().load()
+
+    for k, v in encoding.items():
             if k in val1:
                 val1[k].encoding = v
 
-        obj_size1 = len(tu.processing.write_pkl_zstd(val1.to_netcdf()))
-        obj_sizes.append(obj_size1)
+    obj_sizes = {}
+    for l in range(block_size):
+        # print(l)
+        lons1 = lons[:(l+1)]
+        lats1 = lats[:(l+1)]
+
+        val2 = val1.sel(lon=lons1, lat=lats1)
+        obj_size1 = len(tu.processing.write_pkl_zstd(val2.to_netcdf()))
+
+        obj_sizes.update({l+1: obj_size1})
 
     return obj_sizes
 
@@ -197,35 +219,47 @@ class Processor(object):
         return dataset_list
 
 
-    def load_raw_grid(self, data_path, lon_dim_name, lat_dim_name, time_dim_name=None, chunks=None, parallel=True, preprocessor=None, postprocessor=None, **kwargs):
+    def parse_paths(self, glob_str, date_format=None, from_date=None, to_date=None):
+        """
+
+        """
+        paths = parse_data_paths(glob_str, date_format, from_date, to_date)
+
+        self.source_data_paths = paths
+
+        return paths
+
+
+    def load_raw_grid(self, lon_dim_name, lat_dim_name, time_dim_name=None, chunks=None, parallel=True, preprocessor=None, postprocessor=None, **kwargs):
         """
         The processor takes a str path to one or many files that xr.open_mfdataset can open (e.g. netcdf) and runs a preprocessor and postprocessor (optionally) to provide the proper dimensions and names for the dataset. A Dataset can also be passed to data_path and this will skip the preprocessing step.
         """
         ## initial loading and processing
-        if isinstance(data_path, str):
-            grid1 = xr.open_mfdataset(data_path, chunks=chunks, parallel=parallel, preprocess=preprocessor)
+        # if isinstance(data_path, str):
+        #     grid1 = xr.open_mfdataset(data_path, chunks=chunks, parallel=parallel, preprocess=preprocessor)
 
-        elif isinstance(data_path, list):
-            if isinstance(data_path[0], str):
-                paths = []
-                [paths.extend(glob(p)) for p in data_path]
-                paths.sort()
-                grid1 = xr.open_mfdataset(paths, chunks=chunks, parallel=parallel, preprocess=preprocessor)
-                data_path = paths.copy()
-            else:
-                raise TypeError('If data_path is a list, then it must be a list of str paths.')
+        # elif isinstance(data_path, list):
+        #     if isinstance(data_path[0], str):
+        #         paths = []
+        #         [paths.extend(glob(p)) for p in data_path]
+        #         paths.sort()
+        #         grid1 = xr.open_mfdataset(paths, chunks=chunks, parallel=parallel, preprocess=preprocessor)
+        #         data_path = paths.copy()
+        #     else:
+        #         raise TypeError('If data_path is a list, then it must be a list of str paths.')
 
-        elif isinstance(data_path, xr.Dataset):
-            grid1 = data_path
-        else:
-            raise TypeError('data_path must be a str path that xr.open_mfdataset can open or an xr.Dataset.')
+        # elif isinstance(data_path, xr.Dataset):
+        #     grid1 = data_path
+        # else:
+        #     raise TypeError('data_path must be a str path that xr.open_mfdataset can open or an xr.Dataset.')
+
+        grid1 = xr.open_mfdataset(self.source_data_paths, chunks=chunks, parallel=parallel, preprocess=preprocessor)
 
         if isinstance(postprocessor, types.FunctionType):
             grid1 = postprocessor(grid1, **kwargs)
 
         ## add attributes
         self._dims = grid1.dims
-        self._source_data_path = data_path
         self._lon_dim_name = lon_dim_name
         self._lat_dim_name = lat_dim_name
         self._time_dim_name = time_dim_name
@@ -250,7 +284,7 @@ class Processor(object):
     #     self._extract_results_distribution()
 
 
-    def save_grid_chunks(self, output_path, xy_size=None, xy_size_multiplier=10):
+    def save_grid_chunks(self, output_path, block_size=None, block_size_multiplier=10):
         """
 
         """
@@ -265,17 +299,17 @@ class Processor(object):
             self._extract_null_grid()
 
         ## Parameters
-        if xy_size is None:
+        if block_size is None:
             obj_dict = self._obj_dict.copy()
 
-            xy_chunk_size = int(obj_dict['xy_size'] * xy_size_multiplier)
-        elif isinstance(xy_size, int):
-            xy_chunk_size = int(xy_size * xy_size_multiplier)
+            xy_chunk_size = int(obj_dict['block_size'] * block_size_multiplier)
+        elif isinstance(block_size, int):
+            xy_chunk_size = int(block_size * block_size_multiplier)
 
         chunks = {self._time_dim_name: self._dims['time'], self._lon_dim_name: xy_chunk_size, self._lat_dim_name: xy_chunk_size}
 
         ## Open grid
-        grid1 = self.load_raw_grid(self._source_data_path, self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks=chunks, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor, **self._kwargs)
+        grid1 = self.load_raw_grid(self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks=chunks, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor, **self._kwargs)
 
         ## Create the blocks
         block_list = split_grid(grid1, x_size=xy_chunk_size, y_size=xy_chunk_size, n_intervals=None, x_name='lon', y_name='lat', mbytes_size=1600)
@@ -320,7 +354,7 @@ class Processor(object):
         grid1 = None
 
 
-    def _select_sample_points(self, samples=900):
+    def _select_sample_points(self, ds_vars, block_size=10):
         """
 
         """
@@ -329,15 +363,16 @@ class Processor(object):
         else:
             chunks = None
 
-        grid1 = self.load_raw_grid(self._source_data_path, self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks=chunks, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor, **self._kwargs)
-        pts_df = select_sample_points(grid1, samples=samples)
+        grid1 = self.load_raw_grid(self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks=chunks, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor, **self._kwargs)
+        grid1 = grid1[ds_vars]
+        lons_lats = select_sample_points(grid1, block_size=block_size)
 
-        self.sample_points = pts_df
+        self.sample_lons_lats = lons_lats
 
         grid1.close()
         grid1 = None
 
-        return pts_df
+        return lons_lats
 
 
     def _extract_null_grid(self):
@@ -349,7 +384,7 @@ class Processor(object):
         else:
             chunks = None
 
-        grid1 = self.load_raw_grid(self._source_data_path, self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks=chunks, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor, **self._kwargs)
+        grid1 = self.load_raw_grid(self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks=chunks, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor, **self._kwargs)
 
         grid2 = grid1.isel(time=0, height=0, drop=True).copy()
 
@@ -360,83 +395,89 @@ class Processor(object):
         self._null_grid = grid3
 
 
-    def determine_grid_block_size(self, dataset_id, min_size=1000, max_size=1600):
+    def determine_grid_block_size(self, min_size=1000, max_size=2000, block_size=10):
         """
 
         """
         ## checks
-        if not dataset_id in self.datasets:
-            raise ValueError('dataset_id not in datasets.')
+        # if not dataset_id in self.datasets:
+        #     raise ValueError('dataset_id not in datasets.')
 
         grid_checks(self._dims)
 
         ## some nice processing
         self._extract_null_grid()
 
-        ## parameters
+        ## iterate datasets
+        ds_obj_sizes = {}
+
         dataset_list = copy.deepcopy(self.dataset_list)
-        dataset = self.datasets[dataset_id]
-        dataset_enc = dataset['properties']['encoding']
-        encoding = copy.deepcopy(base_encoding)
-        encoding.update(dataset_enc)
+        for dataset in dataset_list:
+            dataset_id = dataset['dataset_id']
+            print(dataset_id)
 
-        ds_vars = [list(ds['properties']['encoding'].keys())  for ds in self.dataset_list if ds['dataset_id'] == dataset_id][0]
+            dataset = self.datasets[dataset_id]
+            dataset_enc = dataset['properties']['encoding']
+            encoding = copy.deepcopy(base_encoding)
+            encoding.update(dataset_enc)
 
-        ## get sample points
-        pts_df = self._select_sample_points(4)
+            ds_vars = [list(ds['properties']['encoding'].keys())  for ds in self.dataset_list if ds['dataset_id'] == dataset_id][0]
 
-        # calc obj sizes
-        grid1 = self.load_raw_grid(self._source_data_path, self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks={self._time_dim_name: self._dims['time'], self._lon_dim_name: 1, self._lat_dim_name: 1}, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor, **self._kwargs)
+            ## get sample points
+            lons_lats = self._select_sample_points(ds_vars, block_size)
 
-        grid2 = grid1[ds_vars]
+            # calc obj sizes
+            grid1 = self.load_raw_grid(self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks={self._time_dim_name: self._dims['time'], self._lon_dim_name: 1, self._lat_dim_name: 1}, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor, **self._kwargs)
 
-        # Do a round of 4 for initial assessment
-        obj_sizes = iter_obj_sizes(pts_df, grid2, encoding)
+            grid2 = grid1[ds_vars]
 
-        null_grid = self._null_grid
-        n_stns = int(null_grid.sum())
-        mean_size = int(sum(obj_sizes)/4)
+            # Do a round of 4 for initial assessment
+            obj_sizes = iter_obj_sizes(lons_lats, grid2, encoding)
 
-        if (sum(obj_sizes)/2 > min_size*1000) or (sum(obj_sizes) > min_size*1000):
-            xy_size = 1
+            null_grid = self._null_grid
+            n_stns = int(null_grid.sum())
 
-            [ds.update({'grouping': 'none'}) for ds in dataset_list]
+            if (obj_sizes[1] > max_size*1000):
+                block_size = 1
+                mean_size = obj_sizes[1]
+                obj_size = obj_sizes[1]
 
-            print('The results for this dataset should not be grouped and has been changed internally.')
+                [ds.update({'grouping': 'none'}) for ds in dataset_list]
 
-        else:
-            xy_size = int(((max_size*1000)//mean_size)**0.5)
-            samples1 = (xy_size+1)**2
+                print('The results for this dataset should not be grouped and has been changed internally.')
 
-            print('Initial xy size estimate is ' + str(xy_size) + '. More samples will be taken to verify...')
+            elif (obj_sizes[10] < min_size*1000):
+                mean_size = obj_sizes[10]/100
+                block_size = int(((max_size*1000)//mean_size)**0.5)
+                obj_size = mean_size * block_size**2
 
-            [ds.update({'grouping': 'blocks'}) for ds in dataset_list]
+            else:
 
-            # sizes_sqr = np.arange(1, xy_size+2)**2 - 1
+                obj_sizes1 = {i: o for i, o in obj_sizes.items() if (o >  min_size*1000) and (o <  max_size*1000)}
+                if len(obj_sizes1) == 0:
+                    raise ValueError('No results for the block size. Widen the range of the min and max sizes.')
 
-            pts_df = self._select_sample_points(samples1)
+                block_size = max(obj_sizes1.keys())
+                mean_size = obj_sizes[block_size]/(block_size**2)
+                obj_size = obj_sizes[block_size]
 
-            obj_sizes = iter_obj_sizes(pts_df, grid2, encoding)
+                [ds.update({'grouping': 'blocks'}) for ds in dataset_list]
 
-            mean_size = int(sum(obj_sizes)/len(obj_sizes))
+            print('block size estimate is ' + str(block_size))
 
-            xy_size = int(((max_size*1000)//mean_size)**0.5)
-
-        obj_dict = {'xy_size': xy_size, 'mean_obj_size': mean_size, 'max_obj_size': max(obj_sizes), 'min_obj_size': min(obj_sizes), 'sum_obj_size': mean_size * n_stns, 'n_stations': n_stns}
-        self._obj_dict = obj_dict
-
-        _ = self.load_dataset_metadata(dataset_list)
-
-        grid1.close()
-        grid1 = None
-        grid2.close()
-        grid2 = None
-
-        return obj_dict
+            obj_dict = {'block_size': block_size, 'obj_size': obj_size, 'sum_obj_sizes': int(mean_size * n_stns), 'n_stations': n_stns}
+            self._obj_dict = obj_dict
 
+            _ = self.load_dataset_metadata(dataset_list)
 
+            grid1.close()
+            grid1 = None
+            grid2.close()
+            grid2 = None
 
+            ds_obj_sizes.update({dataset_id: obj_dict})
 
+        return ds_obj_sizes
 
 
 class Grid(Titan):
@@ -499,38 +540,51 @@ class Grid(Titan):
         return dataset_list
 
 
-    def load_results(self, results, xy_size, dataset_id=None, sum_closed='right', other_closed='left', discrete=True, other_attrs=None, other_encoding=None, run_date=None, skip_resampling=True):
+    def parse_paths(self, glob_str, date_format=None, from_date=None, to_date=None):
+        """
+
+        """
+        paths = parse_data_paths(glob_str, date_format, from_date, to_date)
+
+        self.source_data_paths = paths
+
+        return paths
+
+
+    def load_results(self, block_size, results=None, dataset_id=None, sum_closed='right', other_closed='left', discrete=True, other_attrs=None, other_encoding=None, run_date=None, skip_resampling=True):
         """
 
         """
         ## Open datasets
-        paths = None
         if isinstance(results, list):
-            if isinstance(results[0], str):
-                paths = []
-                [paths.extend(glob(p)) for p in results]
-                paths.sort()
-                data_list = [xr.open_dataset(p) for p in paths if isinstance(p, str)]
-            elif isinstance(results[0], xr.Dataset):
+            # if isinstance(results[0], str):
+            #     paths = []
+            #     [paths.extend(glob(p)) for p in results]
+            #     paths.sort()
+            #     data_list = [xr.open_dataset(p) for p in paths if isinstance(p, str)]
+            if isinstance(results[0], xr.Dataset):
                 data_list = [xr.open_dataset(p) for p in results if isinstance(p, xr.Dataset)]
             else:
-                raise TypeError('If results is a list, then it must be either a list of paths or xr.Datasets.')
-        elif isinstance(results, str):
-            paths = sorted(glob(results))
-            data_list = [xr.open_dataset(p) for p in paths]
+                raise TypeError('If results is a list, then it must be xr.Datasets.')
+        # elif isinstance(results, str):
+        #     paths = sorted(glob(results))
+        #     data_list = [xr.open_dataset(p) for p in paths]
         elif isinstance(results, xr.Dataset):
             data_list = [results]
         else:
-            raise TypeError('results must be a str, list of str/xr.Dataset, or a xr.Dataset.')
+            try:
+                data_list = [xr.open_dataset(p) for p in self.source_data_paths]
+            except:
+                raise TypeError('results must be a list of str/xr.Dataset, a xr.Dataset, or use the method parse_paths.')
 
         ## Determine dataset_ids
         if isinstance(dataset_id, str):
             data_dict = {dataset_id: data_list}
         else:
-            if isinstance(paths, list):
-                ds_ids = set([os.path.split(p)[1].split('_part_')[0] for p in paths])
+            if hasattr(self, 'source_data_paths'):
+                ds_ids = set([os.path.split(p)[1].split('_part_')[0] for p in self.source_data_paths])
                 data_dict = {ds: [] for ds in ds_ids}
-                for i, p in enumerate(paths):
+                for i, p in enumerate(self.source_data_paths):
                     ds_id = os.path.split(p)[1].split('_part_')[0]
                     data_dict[ds_id].append(data_list[i])
             else:
@@ -551,7 +605,7 @@ class Grid(Titan):
                 data.close()
                 data = None
 
-                blocks = split_grid(data1, xy_size, xy_size)
+                blocks = split_grid(data1, block_size, block_size)
                 data1.close()
                 data1 = None
 
