@@ -5,6 +5,7 @@ Created on Sat May  1 13:22:21 2021
 
 @author: mike
 """
+import os
 import socket
 import numpy as np
 import zstandard as zstd
@@ -15,6 +16,8 @@ import ssl
 import requests
 import orjson
 from email.message import EmailMessage
+from glob import glob
+from datetime import date, datetime
 
 ############################################
 ### Parameters
@@ -25,6 +28,54 @@ from email.message import EmailMessage
 ### Functions
 
 
+def path_date_parser(path, date_format=None):
+    """
+
+    """
+    name1 = os.path.splitext(os.path.split(path)[1])[0]
+    date1 = pd.to_datetime(name1, format=date_format, exact=False, infer_datetime_format=True)
+
+    return date1
+
+
+def filter_data_paths(paths, date_format=None, from_date=None, to_date=None, return_dates=False):
+    """
+
+    """
+    # if isinstance(glob_str, str):
+    #     paths = glob(glob_str)
+    dates = pd.DataFrame([[p, path_date_parser(p, date_format=date_format)] for p in paths], columns=['path', 'date'])
+
+    if isinstance(from_date, (str, pd.Timestamp, date, datetime)):
+        dates = dates[dates['date'] >= pd.Timestamp(from_date)].copy()
+    if isinstance(to_date, (str, pd.Timestamp, date, datetime)):
+        dates = dates[dates['date'] <= pd.Timestamp(to_date)].copy()
+
+    if not return_dates:
+        dates = dates['path'].tolist()
+
+    return dates
+
+
+def parse_data_paths(glob_obj, date_format=None, from_date=None, to_date=None, return_dates=False):
+    """
+
+    """
+    if isinstance(glob_obj, list):
+        paths = []
+        [paths.extend(glob(p)) for p in glob_obj]
+        paths.sort()
+    elif isinstance(glob_obj, str):
+        paths = glob(glob_obj)
+
+    if (from_date is not None) or (to_date is not None):
+        paths = filter_data_paths(paths, date_format, from_date, to_date, return_dates)
+
+    paths.sort()
+
+    return paths
+
+
 def make_run_date_key(run_date=None):
     """
 
