@@ -770,14 +770,15 @@ def assign_station_ids_df(stns_df, extent=False, precision=5):
 #     return stns3
 
 
-def process_sparse_stations_from_df(stns, dataset_id=None, remote=None):
+def process_sparse_stations_from_df(stns, dataset_id=None, connection_config=None, bucket=None, version=2):
     """
     Function that takes a stns dataframe of station data and converts it to an Xarray Dataset for Tethys. This is ultimately meant to be combined with the time series data for futher processing. If a geometry column is provided, it must be as a geojson-type dict (not a geopandas column).
 
     """
     ## Get existing stations from tethys
-    if isinstance(dataset_id, str) and isinstance(remote, dict):
+    if isinstance(dataset_id, str) and isinstance(connection_config, (dict, str)) and isinstance(bucket, str):
         try:
+            remote = {'connection_config': connection_config, 'bucket': bucket, 'version': version}
             tethys = Tethys([remote])
             remote_stns = tethys.get_stations(dataset_id)
             remote_stns1 = pd.DataFrame(remote_stns)[['station_id', 'ref']]
@@ -1232,20 +1233,24 @@ def stats_for_dataset_metadata(stns):
     """
     I need time_range, extent, and if grid the spatial_resolution.
     """
+    dict1 = {}
     ## spatial resolution
-    lat_dim = int(np.median([s['dimensions']['lat'] for s in stns]))
+    if 'lat' in stns[0]['dimensions']:
+        lat_dim = int(np.median([s['dimensions']['lat'] for s in stns]))
 
-    type1 = stns[0]['geometry']['type']
+        type1 = stns[0]['geometry']['type']
 
-    if type1 in ['Polygon', 'Line']:
-        geo = [s['geometry']['coordinates'][0][0][-1] for s in stns]
-    else:
-        geo = [s['geometry']['coordinates'][-1] for s in stns]
+        if type1 in ['Polygon', 'Line']:
+            geo = [s['geometry']['coordinates'][0][0][-1] for s in stns]
+        else:
+            geo = [s['geometry']['coordinates'][-1] for s in stns]
 
-    geo1 = np.array(geo).round(5)
-    geo1.sort()
-    diff1 = np.diff(geo1)
-    res1 = round(np.median(diff1[diff1 > 0])/lat_dim, 5)
+        geo1 = np.array(geo).round(5)
+        geo1.sort()
+        diff1 = np.diff(geo1)
+        res1 = round(np.median(diff1[diff1 > 0])/lat_dim, 5)
+
+        dict1.update({'spatial_resolution': res1})
 
     ## Extent
     geo = np.array([s['geometry']['coordinates'] for s in stns]).round(5)
@@ -1257,6 +1262,8 @@ def stats_for_dataset_metadata(stns):
 
     extent1 = mapping(box(min_lon, min_lat, max_lon, max_lat))
 
+    dict1.update({'extent': extent1})
+
     ## time range
     trange1 = np.array([[s['time_range']['from_date'], s['time_range']['to_date']] for s in stns])
     mins, maxes = trange1.T
@@ -1264,14 +1271,15 @@ def stats_for_dataset_metadata(stns):
     min_t = min(mins)
     max_t = max(maxes)
 
+    dict1.update({'time_range': {'from_date': min_t, 'to_date': max_t}})
+
     ## Heights
     heights1 = []
     [heights1.extend(s['heights']) for s in stns]
     heights2 = list(set(heights1))
     heights2.sort()
 
-    ## Make the dict output
-    dict1 = {'spatial_resolution': res1, 'extent': extent1, 'time_range': {'from_date': min_t, 'to_date': max_t}, 'heights': heights2}
+    dict1.update({'heights': heights2})
 
     return dict1
 
