@@ -19,6 +19,7 @@ from tethys_utils.misc import make_run_date_key, write_json_zstd, write_pkl_zstd
 from tethys_utils.processing import compare_xrs, process_station_summ, stats_for_dataset_metadata
 # from processing import compare_xrs, process_station_summ, stats_for_dataset_metadata
 from datetime import date, datetime
+from tethysts import Tethys
 
 ############################################
 ### Parameters
@@ -159,7 +160,8 @@ def put_remote_dataset(s3, bucket, dataset, stns, run_date=None, version=3):
     ds_stats = stats_for_dataset_metadata(stns)
 
     if dataset['spatial_distribution'] == 'sparse':
-        ds_stats.pop('spatial_resolution')
+        if 'spatial_resolution' in ds_stats:
+            ds_stats.pop('spatial_resolution')
 
     dataset.update(ds_stats)
 
@@ -436,98 +438,100 @@ def update_results_s3(processing_code, data_dict, run_date_dict, remote, threads
     for ds_id, results in data_dict.items():
         print('--dataset_id: ' + ds_id)
 
-        run_date_key = run_date_dict[ds_id]
+        if len(results) > 0:
 
-        # Create the Key info dict
-        prefix = key_patterns[version]['results'].split('{station_id}')[0].format(dataset_id=ds_id)
+            run_date_key = run_date_dict[ds_id]
 
-        keys1 = list_objects_s3(s3, bucket, prefix)
-        obj_df1 = keys1[keys1.Key.str.contains('results.nc')].copy()
-        # obj_df1['dataset_id'] = obj_df1['Key'].apply(lambda x: x.split('/')[2])
-        obj_df1['station_id'] = obj_df1['Key'].apply(lambda x: x.split('/')[3])
+            # Create the Key info dict
+            prefix = key_patterns[version]['results'].split('{station_id}')[0].format(dataset_id=ds_id)
 
-        last_date1 = obj_df1.groupby(['station_id'])['KeyDate'].last().reset_index()
+            keys1 = list_objects_s3(s3, bucket, prefix)
+            obj_df1 = keys1[keys1.Key.str.contains('results.nc')].copy()
+            # obj_df1['dataset_id'] = obj_df1['Key'].apply(lambda x: x.split('/')[2])
+            obj_df1['station_id'] = obj_df1['Key'].apply(lambda x: x.split('/')[3])
+
+            last_date1 = obj_df1.groupby(['station_id'])['KeyDate'].last().reset_index()
 
 
-        def update_result(result):
-            """
+            def update_result(result):
+                """
 
-            """
-            ## Process data
-            try:
-                new1 = xr.load_dataset(read_pkl_zstd(result, False))
-            except:
-                print('Data could not be opened')
-                return None
+                """
+                ## Process data
+                try:
+                    new1 = xr.load_dataset(read_pkl_zstd(result, False))
+                except:
+                    print('Data could not be opened')
+                    return None
 
-            stn_id = str(new1.squeeze()['station_id'].values)
-            # print('station_id: ' + stn_id)
+                stn_id = str(new1.squeeze()['station_id'].values)
+                # print('station_id: ' + stn_id)
 
-            vars1 = list(new1.variables)
-            parameter = [v for v in vars1 if 'dataset_id' in new1[v].attrs][0]
-            result_attrs = new1[parameter].attrs.copy()
+                vars1 = list(new1.variables)
+                parameter = [v for v in vars1 if 'dataset_id' in new1[v].attrs][0]
+                result_attrs = new1[parameter].attrs.copy()
 
-            ds_id = result_attrs['dataset_id']
-            mod_date_key = new1.attrs['history'].split(':')[0]
+                ds_id = result_attrs['dataset_id']
+                mod_date_key = new1.attrs['history'].split(':')[0]
 
-            if no_comparison:
-                up1 = new1
-            else:
-                last_date_key_df = last_date1[last_date1['station_id'] == stn_id]
-                if last_date_key_df.empty:
-                    last_date_key = None
+                if no_comparison:
+                    up1 = new1
                 else:
-                    last_date_key = make_run_date_key(last_date_key_df['KeyDate'].iloc[0])
+                    last_date_key_df = last_date1[last_date1['station_id'] == stn_id]
+                    if last_date_key_df.empty:
+                        last_date_key = None
+                    else:
+                        last_date_key = make_run_date_key(last_date_key_df['KeyDate'].iloc[0])
 
-                up1 = compare_datasets_from_s3(conn_config, bucket, new1, add_old=add_old, last_run_date_key=last_date_key, public_url=public_url, version=version)
+                    up1 = compare_datasets_from_s3(conn_config, bucket, new1, add_old=add_old, last_run_date_key=last_date_key, public_url=public_url, version=version)
 
-            ## Save results
-            if isinstance(up1, xr.Dataset) and (len(up1[parameter].time) > 0):
+                ## Save results
+                if isinstance(up1, xr.Dataset) and (len(up1[parameter].time) > 0):
 
-                # print('Save results')
-                key_dict = {'dataset_id': ds_id, 'station_id': stn_id, 'run_date': run_date_key}
+                    # print('Save results')
+                    key_dict = {'dataset_id': ds_id, 'station_id': stn_id, 'run_date': run_date_key}
 
-                new_key = key_patterns[version]['results'].format(**key_dict)
+                    new_key = key_patterns[version]['results'].format(**key_dict)
 
-                cctx = zstd.ZstdCompressor(level=1)
-                c_obj = cctx.compress(up1.to_netcdf())
+                    cctx = zstd.ZstdCompressor(level=1)
+                    c_obj = cctx.compress(up1.to_netcdf())
 
-                obj_resp = s3.put_object(Body=c_obj, Bucket=bucket, Key=new_key, ContentType='application/zstd', Metadata={'run_date': mod_date_key})
+                    obj_resp = s3.put_object(Body=c_obj, Bucket=bucket, Key=new_key, ContentType='application/zstd', Metadata={'run_date': mod_date_key})
 
-                ## Process stn data
-                # print('Save station data')
+                    ## Process stn data
+                    # print('Save station data')
 
-                ## Process object key infos
-                stn_obj_df1 = obj_df1[obj_df1['station_id'] == stn_id].drop(['LastModified', 'station_id'], axis=1).copy()
-                keys = stn_obj_df1['Key'].unique()
-                new_key_info = [new_key, obj_resp['ResponseMetadata']['HTTPHeaders']['etag'].replace('"', ''), len(c_obj), pd.Timestamp(run_date_key).tz_localize(None)]
+                    ## Process object key infos
+                    stn_obj_df1 = obj_df1[obj_df1['station_id'] == stn_id].drop(['LastModified', 'station_id'], axis=1).copy()
+                    keys = stn_obj_df1['Key'].unique()
+                    new_key_info = [new_key, obj_resp['ResponseMetadata']['HTTPHeaders']['etag'].replace('"', ''), len(c_obj), pd.Timestamp(run_date_key).tz_localize(None)]
+
+                    if new_key in keys:
+                        stn_obj_df1[stn_obj_df1['Key'] == new_key] = new_key_info
+                    else:
+                        stn_obj_df1.loc[len(stn_obj_df1)] = new_key_info
+
+                    info1 = [S3ObjectKey(key=row['Key'], bucket=bucket, content_length=row['Size'], etag=row['ETag'], run_date=row['KeyDate']) for i, row in stn_obj_df1.iterrows()]
+
+                    ## Final station processing and saving
+                    stn_m = process_station_summ(up1, info1, mod_date=mod_date_key)
+
+                    up_stns = put_remote_station(s3, bucket, stn_m, run_date=mod_date_key, version=version)
 
-                if new_key in keys:
-                    stn_obj_df1[stn_obj_df1['Key'] == new_key] = new_key_info
                 else:
-                    stn_obj_df1.loc[len(stn_obj_df1)] = new_key_info
+                    print('No new data to update')
 
-                info1 = [S3ObjectKey(key=row['Key'], bucket=bucket, content_length=row['Size'], etag=row['ETag'], run_date=row['KeyDate']) for i, row in stn_obj_df1.iterrows()]
-
-                ## Final station processing and saving
-                stn_m = process_station_summ(up1, info1, mod_date=mod_date_key)
-
-                up_stns = put_remote_station(s3, bucket, stn_m, run_date=mod_date_key, version=version)
-
-            else:
-                print('No new data to update')
-
-            ## Get rid of big objects
-            new1 = None
-            up1 = None
+                ## Get rid of big objects
+                new1 = None
+                up1 = None
 
 
-        ## Run the threadpool
-        # output = ThreadPool(threads).imap_unordered(update_result, results)
-        with ThreadPool(threads) as pool:
-            output = pool.map(update_result, results)
-            pool.close()
-            pool.join()
+            ## Run the threadpool
+            # output = ThreadPool(threads).imap_unordered(update_result, results)
+            with ThreadPool(threads) as pool:
+                output = pool.map(update_result, results)
+                pool.close()
+                pool.join()
 
 
 def delete_result_objects_s3(conn_config, bucket, dataset_ids=None, keep_last=10, threads=50, version=3):
@@ -638,7 +642,8 @@ def process_run_date(processing_code, dataset_list, connection_config, bucket, r
             key1 = key_patterns[version]['stations'].format(dataset_id=dataset_id)
             obj1 = get_object_s3(key1, connection_config, bucket, counter=0)
             rok1 = read_json_zstd(obj1)
-            rok2 = pd.DataFrame([r['results_object_key'] for r in rok1])
+            rok1b = [r['results_object_key'] if isinstance(r['results_object_key'], dict) else r['results_object_key'][-1] for r in rok1]
+            rok2 = pd.DataFrame(rok1b)
             rok2['run_date'] = pd.to_datetime(rok2['run_date'])
             obj_list = rok2.dropna(subset=['run_date'])
             last_run_date = obj_list['run_date'].max()
@@ -662,13 +667,92 @@ def process_run_date(processing_code, dataset_list, connection_config, bucket, r
     return run_date_dict
 
 
+def reprocess_datasets(dataset_id, conn_config, bucket, public_url, threads=20):
+    """
+
+    """
+    tethys = Tethys([{'connection_config': public_url, 'bucket': bucket, 'version': 2}])
+
+    stns = tethys.get_stations(dataset_id, results_object_keys=True)
+
+    ds = tethys._datasets[dataset_id]
+
+    bad_stns = [s for s in stns if 'time_range' not in s]
+
+    prefix = key_patterns[2]['results'].split('{station_id}')[0].format(dataset_id=dataset_id)
+
+    s3 = s3_connection(conn_config)
+    keys1 = list_objects_s3(s3, bucket, prefix)
+    obj_df1 = keys1[keys1.Key.str.contains('results.nc')].copy()
+    obj_df1['station_id'] = obj_df1['Key'].apply(lambda x: x.split('/')[3])
+
+
+    def reprocess_dataset(stn):
+        """
+
+        """
+        stn_id = stn['station_id']
+        run_dates = tethys.get_run_dates(dataset_id, stn['station_id'])
+
+        for run_date in run_dates:
+
+        # run_date = stn['results_object_key']['run_date']
+            run_date_key = make_run_date_key(run_date)
+            mod_date_key = make_run_date_key()
+            up1 = tethys.get_results(dataset_id, stn_id, run_date=run_date, output='Dataset')
+
+            key_dict = {'dataset_id': dataset_id, 'station_id': stn_id, 'run_date': run_date_key}
+
+            new_key = key_patterns[2]['results'].format(**key_dict)
+
+            cctx = zstd.ZstdCompressor(level=1)
+            c_obj = cctx.compress(up1.to_netcdf())
+
+            obj_resp = s3.put_object(Body=c_obj, Bucket=bucket, Key=new_key, ContentType='application/zstd', Metadata={'run_date': mod_date_key})
+
+            ## Process stn data
+            # print('Save station data')
+
+        ## Process object key infos
+        stn_obj_df1 = obj_df1[obj_df1['station_id'] == stn_id].drop(['LastModified', 'station_id'], axis=1).copy()
+        keys = stn_obj_df1['Key'].unique()
+        new_key_info = [new_key, obj_resp['ResponseMetadata']['HTTPHeaders']['etag'].replace('"', ''), len(c_obj), pd.Timestamp(run_date_key).tz_localize(None)]
+
+        if new_key in keys:
+            stn_obj_df1[stn_obj_df1['Key'] == new_key] = new_key_info
+        else:
+            stn_obj_df1.loc[len(stn_obj_df1)] = new_key_info
+
+        info1 = [S3ObjectKey(key=row['Key'], bucket=bucket, content_length=row['Size'], etag=row['ETag'], run_date=row['KeyDate']) for i, row in stn_obj_df1.iterrows()]
+
+        ## Final station processing and saving
+        stn_m = process_station_summ(up1, info1, mod_date=mod_date_key)
+
+        up_stns = put_remote_station(s3, bucket, stn_m, run_date=mod_date_key, version=2)
+
+
+    if len(bad_stns) > 0:
+        print('Updating results and stations...')
+        output = ThreadPool(threads).map(reprocess_dataset, bad_stns)
+
+        print('Updating aggregates...')
+
+        ds_stations = put_remote_agg_stations(s3, bucket, dataset_id, threads*2, version=2)
+        if ds_stations is not None:
+            ds_new = put_remote_dataset(s3, bucket, ds, ds_stations, version=2)
+
+        # Aggregate all datasets for the bucket
+        ds_all = put_remote_agg_datasets(s3, bucket, threads*2, version=2)
+
+    else:
+        print('Nothing to update.')
+
+    print('Finished')
 
 
 
-
-
-
-
+# for ds in titan.dataset_list:
+#     reprocess_datasets(ds['dataset_id'], conn_config, bucket, public_url, threads=20)
 
 
 
